import os
import shutil
import subprocess
import tempfile
import unittest

from click.testing import CliRunner

from pre_commit_vauxoo.cli import main


class TestPreCommitVauxoo(unittest.TestCase):
    def setUp(self):
        super().setUp()
        self.old_environ = os.environ.copy()
        self.original_work_dir = os.getcwd()
        self.tmp_dir = tempfile.mkdtemp(suffix="_pre_commit_vauxoo")
        os.chdir(self.tmp_dir)
        self.runner = CliRunner()
        src_path = os.path.join(os.path.abspath(os.path.dirname(__file__)), "..", "resources")
        self.create_dummy_repo(src_path, self.tmp_dir)
        self.maxDiff = None

    def create_dummy_repo(self, src_path, dest_path):
        subprocess.call(["git", "init", "--initial-branch=main", dest_path])
        dest_subpath = os.path.join(dest_path, os.path.basename(src_path))
        shutil.copytree(src_path, dest_subpath)
        subprocess.call(["git", "add", dest_subpath])

    def tearDown(self):
        super().tearDown()
        # change to original work dir
        os.chdir(self.original_work_dir)
        # Cleanup temporary files
        if os.path.isdir(self.tmp_dir) and self.tmp_dir != "/":
            shutil.rmtree(self.tmp_dir, ignore_errors=True)
        # reset environment variables
        os.environ.clear()
        os.environ.update(self.old_environ)

    def test_basic(self):
        os.environ["INCLUDE_LINT"] = "resources"
        os.environ["PRECOMMIT_HOOKS_TYPE"] = "all"
        result = self.runner.invoke(main, [])
        self.assertEqual(result.exit_code, 0, "Exited with error %s" % result)
        with open(os.path.join(self.tmp_dir, "pyproject.toml"), "r") as f_pyproject:
            self.assertIn("skip-string-normalization=false", f_pyproject, "Skip string normalization not set")

    def test_chdir(self):
        self.runner = CliRunner()
        os.environ["PRECOMMIT_HOOKS_TYPE"] = "all"
        os.chdir("resources")
        result = self.runner.invoke(main, [])
        self.assertEqual(result.exit_code, 0, "Exited with error %s" % result)

    def test_exclude_lint_path(self):
        self.runner = CliRunner()
        os.chdir("resources")
        os.environ["PRECOMMIT_HOOKS_TYPE"] = "all"
        os.environ["BLACK_SKIP_STRING_NORMALIZATION"] = "false"
        os.environ["EXCLUDE_LINT"] = "resources/module_example1/models"
        result = self.runner.invoke(main, [])
        self.assertEqual(result.exit_code, 0, "Exited with error %s" % result)
        with open(os.path.join(self.tmp_dir, "pyproject.toml"), "r") as f_pyproject:
            self.assertIn("skip-string-normalization=false", f_pyproject, "Skip string normalization not set")

    def test_disable_lints(self):
        self.runner = CliRunner()
        os.environ["DISABLE_PYLINT_CHECKS"] = "import-error"
        result = self.runner.invoke(main, [])
        self.assertEqual(result.exit_code, 0, "Exited with error %s" % result)

    def test_exclude_autofix(self):
        self.runner = CliRunner()
        os.chdir("resources")
        os.environ["PRECOMMIT_HOOKS_TYPE"] = "all"
        os.environ["EXCLUDE_AUTOFIX"] = "resources/module_example1/demo/"
        os.environ["BLACK_SKIP_STRING_NORMALIZATION"] = "true"
        result = self.runner.invoke(main, [])
        self.assertEqual(result.exit_code, 0, "Exited with error %s" % result)
        with open(os.path.join(self.tmp_dir, "pyproject.toml"), "r") as f_pyproject:
            self.assertIn("skip-string-normalization=true", f_pyproject, "Skip string normalization not set")

    def test_fail_warning(self):
        os.environ["PRECOMMIT_FAIL_OPTIONAL"] = "1"
        # Only optional
        os.environ["PRECOMMIT_HOOKS_TYPE"] = "optional"
        result = self.runner.invoke(main, [])
        self.assertEqual(result.exit_code, 1, "Exited without error")

    def test_rm_options(self):
        # Only mandatory
        os.environ["PRECOMMIT_HOOKS_TYPE"] = "all,-optional,-fix,-experimental"
        result = self.runner.invoke(main, [])
        self.assertEqual(result.exit_code, 0, "Exited with error %s" % result)

    def test_install_git_hook_pre_commit(self):
        """Test .git/hooks/pre-commit script"""
        git_hook_pre_commit = os.path.join(self.tmp_dir, ".git", "hooks", "pre-commit")
        self.assertFalse(os.path.isfile(git_hook_pre_commit), "File created before to install it")
        result = self.runner.invoke(main, ["--install"])
        self.assertEqual(result.exit_code, 0, "Exited with error %s" % result)
        self.assertTrue(os.path.isfile(git_hook_pre_commit), "File not created")
        with open(git_hook_pre_commit, "r") as f_git_hook_pre_commit:
            self.assertIn("pre-commit-vauxoo", f_git_hook_pre_commit.read(), "File pre-commit not generated correctly")
        os.environ["NOLINT"] = "0"
        exit_code = subprocess.call(
            [
                "git",
                "-c",
                "user.name='Test'",
                "-c",
                "user.email=test@vauxoo.com",
                "commit",
                "--allow-empty",
                "-am",
                "testing",
            ]
        )
        self.assertEqual(exit_code, 0, "Exited with error_code %s" % exit_code)


if __name__ == "__main__":
    unittest.main()



{
    "name": "module_warnings1",
    "summary": """
        Short (1 phrase/line) summary of the module's purpose, used as
        subtitle on modules listing or apps.openerp.com""",
    "author": "Vauxoo,Odoo Community Association (OCA)",
    "license": "AGPL-3",
    "website": "http://www.yourcompany.com",
    # Categories can be used to filter modules in modules listing
    # Check https://github.com/odoo/odoo/blob/15.0/odoo/addons/base/data/ir_module_category_data.xml
    # for the full list
    "category": "Uncategorized",
    "version": "15.0.1.0.0",
    # any module necessary for this one to work correctly
    "depends": ["base"],
    # always loaded
    "data": [
        # 'security/ir.model.access.csv',
        "views/views.xml",
    ],
    # only loaded in demonstration mode
    "demo": [
        "demo/demo.xml",
    ],
}



"""Module that contains the command line app.

Why does this file exist, and why not put this in __main__?

  You might be tempted to import things from __main__ later, but that will cause
  problems: the code will get executed twice:

  - When you run `python -mpre_commit_vauxoo` python will execute
    ``__main__.py`` as a script. That means there won't be any
    ``pre_commit_vauxoo.__main__`` in ``sys.modules``.
  - When you import __main__ it will get executed again (as a module) because
    there's no ``pre_commit_vauxoo.__main__`` in ``sys.modules``.

  Also see (1) from http://click.pocoo.org/5/setuptools/#setuptools-integration
"""


import contextlib
import os
import subprocess

import click

from pre_commit_vauxoo import pre_commit_vauxoo


def source_variables():
    # Overwrite os.environ with variables.sh file only if it was not already defined
    try:
        repo_dirname = pre_commit_vauxoo.get_repo()
    except subprocess.CalledProcessError:
        return
    envdict = pre_commit_vauxoo.envfile2envdict(repo_dirname)
    os.environ.update({var: value for var, value in envdict.items() if var not in os.environ})


@contextlib.contextmanager
def env_clear():
    """Clear environment variables after finished the method"""
    old_environ = os.environ.copy()
    try:
        yield
    finally:
        os.environ.clear()
        os.environ.update(old_environ)


def monkey_patch_make_context():
    """monkey patch to run source variables.sh before to parse the click.options"""
    original_make_context = click.core.BaseCommand.make_context

    def custom_make_context(*args, **kwargs):
        with env_clear():
            source_variables()
            return original_make_context(*args, **kwargs)

    click.core.BaseCommand.make_context = custom_make_context


def strcsv2tuple(strcsv, lower=False):
    if isinstance(strcsv, tuple):
        # Crazy!! but click==8.0.1 is sending "value" transformed
        strcsv = ",".join(strcsv)
    strcsv = strcsv and strcsv.strip() or ""
    if not strcsv:
        return ()
    items = ()
    for item in strcsv.split(","):
        item = item.strip()
        if lower:
            item = item.lower()
        items += (item,)
    return items


class CSVChoice(click.Choice):
    envvar_list_splitter = ","

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.name += " CSV"

    def convert(self, value, param, ctx):
        values = ()
        # case_sensitive parameter is not compatible with click 6.6
        # So we can do it manually here since that we are transforming it
        for v in strcsv2tuple(value, lower=True):
            values += (super().convert(v, param, ctx),)
        return values


class CSVStringParamType(click.types.StringParamType):
    envvar_list_splitter = ","

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.name += " CSV"

    def convert(self, value, param, ctx):
        values = ()
        for v in strcsv2tuple(value):
            values += (super().convert(v, param, ctx),)
        return values


class CSVPath(click.Path):
    envvar_list_splitter = ","

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.name += " CSV"

    def convert(self, value, param, ctx):
        values = ()
        for v in strcsv2tuple(value):
            try:
                new_value = super().convert(v, param, ctx)
            except click.exceptions.BadParameter:
                # The envvar are using path based on root repo path
                new_value = os.path.join(pre_commit_vauxoo.get_repo(), v)
                new_value = super().convert(new_value, param, ctx)
            values += (new_value,)
        return values


def merge_tuples(ctx, param, value):
    """Convert (('value1', 'value2'), ('value3')) to ('value1', 'value2', 'value3')
    It is useful for csv separated by commas parameters but using multiple number of args
    """
    if value is None:
        return value
    values = ()
    for v in value:
        if not isinstance(v, tuple):
            v = (v,)
        values += v
    return values


def precommit_hooks_type_callback(ctx, param, value):
    """If value is 'all' so all the option will be assigned
    If value has prefix '-' so that option will be removed
    value = 'all,-fix' will return 'mandatory,optional,experimental'
    """
    values = merge_tuples(ctx, param, value)
    values = set(values)
    all_values = {i for i in param.type.choices if not i.startswith("-") and not i == "all"}
    if "all" in values:
        values -= {"all"}
        values |= all_values
    for v in values.copy():
        if v.startswith("-"):
            values -= {v, v[1:]}
    return tuple(values)


new_extra_kwargs = {}
try:
    if tuple(map(int, click.__version__.split("."))) >= (7, 0):
        # It is only compatible for click >= 7.0 but it is not a big deal if it is not enabled
        # For record, dockerv image is using click 6.6 version
        new_extra_kwargs["show_envvar"] = True
except (TypeError, ValueError, AttributeError):  # pylint: disable=except-pass
    pass

monkey_patch_make_context()


PRECOMMIT_HOOKS_TYPE = ["mandatory", "optional", "fix", "experimental"]
PRECOMMIT_HOOKS_TYPE += ["all"] + ["-%s" % i for i in PRECOMMIT_HOOKS_TYPE]


@click.command()
# click 6.6 used in dockerv doesn't support to use envvar for click.argument :(
# More info https://github.com/pallets/click/issues/714 workaround using option instead.
@click.option(
    "--paths",
    "-p",
    multiple=True,
    envvar="INCLUDE_LINT",
    type=CSVPath(exists=True),
    callback=merge_tuples,
    default=["."],
    show_default=True,
    help="PATHS are the specific filenames to run hooks on separated by commas.",
    **new_extra_kwargs,
)
@click.option(
    "--no-overwrite",
    envvar="PRECOMMIT_NO_OVERWRITE_CONFIG_FILES",
    type=click.BOOL,
    is_flag=True,
    default=False,
    show_default=True,
    help="Overwrite configuration files. "
    "\f\n*If True, existing configuration files into the project will be overwritten. "
    "\f\n*If False, then current files will be used, if they exist.",
    **new_extra_kwargs,
)
@click.option(
    "--fail-optional",
    envvar="PRECOMMIT_FAIL_OPTIONAL",
    type=click.BOOL,
    default=False,
    is_flag=True,
    show_default=True,
    help="Change the exit_code for 'optional' precommit-hooks-type."
    "\f\n*If this flag is enabled so the exit_code will be -1 (error) if 'optional' fails."
    "\f\n*If it is disabled (by default), exit_code will be 0 (successful) even if 'optional' fails.",
    **new_extra_kwargs,
)
@click.option(
    "--exclude-autofix",
    "-x",
    type=CSVPath(exists=True),
    multiple=True,
    callback=merge_tuples,
    envvar="EXCLUDE_AUTOFIX",
    help="Exclude paths on which to run the autofix pre-commit configuration, separated by commas",
    **new_extra_kwargs,
)
@click.option(
    "--exclude-lint",
    "-l",
    type=CSVPath(exists=True),
    multiple=True,
    callback=merge_tuples,
    envvar="EXCLUDE_LINT",
    help="Paths to exclude checks, separated by commas.",
    **new_extra_kwargs,
)
@click.option(
    "--pylint-disable-checks",
    "-d",
    type=CSVStringParamType(),
    callback=merge_tuples,
    envvar="PYLINT_DISABLE_CHECKS",
    help="Pylint checks to disable, separated by commas.",
    **new_extra_kwargs,
)
@click.option(
    "--skip-string-normalization",
    "-S",
    envvar="BLACK_SKIP_STRING_NORMALIZATION",
    is_flag=True,
    default=False,
    show_default=True,
    help="If '-t fix' is enabled, "
    "don't normalize string quotes or prefixes '' -> \"\""
    "\f\nThis parameter is related to 'black' hook",
    **new_extra_kwargs,
)
@click.option(
    "--precommit-hooks-type",
    "-t",
    type=CSVChoice(PRECOMMIT_HOOKS_TYPE),
    default=["all", "-fix"],
    multiple=True,
    callback=precommit_hooks_type_callback,
    show_default=True,
    envvar="PRECOMMIT_HOOKS_TYPE",
    help="Pre-commit configuration file to run hooks, separated by commas. "
    "\f\nprefix '-' means that the option will be removed. "
    "\f\n*Mandatory: Stable hooks that needs to be fixed (Affecting build status). "
    "\f\n*Optional: Optional hooks that could be fixed later. "
    "(No affects build status almost '--fail-optional' is set). "
    "\f\n*Experimental: Experimental hooks that only to test. (No affects build status). "
    "\f\n*Fix: Hooks auto fixing source code (Affects build status). "
    "\f\n*All: All configuration files to run hooks. ",
    **new_extra_kwargs,
)
@click.option(
    "--install",
    type=click.BOOL,
    is_flag=True,
    default=False,
    help="Install the pre-commit script"
    "\f\nUsing this option a '.git/hooks/pre-commit' will be created"
    "\f\nNow your command 'git commit' will run 'pre-commit-vauxoo' before to commit",
    **new_extra_kwargs,
)
def main(*args, **kwargs):
    """pre-commit-vauxoo run pre-commit with custom validations and configuration files"""
    pre_commit_vauxoo.main(*args, **kwargs)



#!/usr/bin/env python
from __future__ import absolute_import, print_function

import io
import re
from glob import glob
from os.path import basename, dirname, join, splitext

from setuptools import find_packages, setup

try:
    from pbr import git
except ImportError:
    git = None


def generate_changelog():
    fname = "ChangeLog"
    if not git:
        changelog_str = '# ChangeLog was not generated. You need to install "pbr"'
        with open(fname, "w") as fchg:
            fchg.write(changelog_str)
        return changelog_str
    changelog = git._iter_log_oneline()
    changelog = git._iter_changelog(changelog)
    git.write_git_changelog(changelog=changelog)
    # git.generate_authors()
    return read(fname)


def generate_dependencies():
    return read("requirements.txt").splitlines()


def read(*names, **kwargs):
    with io.open(join(dirname(__file__), *names), encoding=kwargs.get("encoding", "utf8")) as fh:
        return fh.read()


setup(
    name="pre-commit-vauxoo",
    version="5.1.2",
    license="LGPL-3.0-or-later",
    description="pre-commit script to run automatically the configuration and variables custom from Vauxoo",
    long_description="{}\n{}".format(
        re.compile("^.. start-badges.*^.. end-badges", re.M | re.S).sub("", read("README.rst")),
        re.sub(":[a-z]+:`~?(.*?)`", r"``\1``", generate_changelog()),
    ),
    author="Vauxoo",
    author_email="info@vauxoo.com",
    url="https://github.com/Vauxoo/pre-commit-vauxoo",
    packages=find_packages("src"),
    package_dir={"": "src"},
    py_modules=[splitext(basename(path))[0] for path in glob("src/*.py")],
    include_package_data=True,
    zip_safe=False,
    classifiers=[
        # complete classifier list: http://pypi.python.org/pypi?%3Aaction=list_classifiers
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: GNU Lesser General Public License v3 or later (LGPLv3+)",
        "Operating System :: Unix",
        "Operating System :: POSIX",
        "Operating System :: Microsoft :: Windows",
        "Operating System :: MacOS",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        # 'Programming Language :: Python :: Implementation :: CPython',
        # 'Programming Language :: Python :: Implementation :: PyPy',
        # uncomment if you test on these interpreters:
        # 'Programming Language :: Python :: Implementation :: IronPython',
        # 'Programming Language :: Python :: Implementation :: Jython',
        # 'Programming Language :: Python :: Implementation :: Stackless',
        "Topic :: Utilities",
    ],
    project_urls={
        "Documentation": "https://pre-commit-vauxoo.readthedocs.io/",
        "Changelog": "https://pre-commit-vauxoo.readthedocs.io/en/latest/changelog.html",
        "Issue Tracker": "https://github.com/Vauxoo/pre-commit-vauxoo/issues",
    },
    keywords=[
        # eg: 'keyword1', 'keyword2', 'keyword3',
    ],
    python_requires=">=3.5",
    install_requires=generate_dependencies(),
    extras_require={
        # eg:
        #   'rst': ['docutils>=0.11'],
        #   ':python_version=="2.6"': ['argparse'],
    },
    entry_points={
        "console_scripts": [
            "pre-commit-vauxoo = pre_commit_vauxoo.cli:main",
        ]
    },
)
